@@ -13,27 +13,33 @@
 # limitations under the License.
 
 import contextlib
+import fnmatch
 import inspect
+import jmespath
 import munch
 import netifaces
 import re
 import six
+import sre_constants
+import sys
 import time
+import uuid
 
 from decorator import decorator
-from neutronclient.common import exceptions as neutron_exc
 
 from shade import _log
 from shade import exc
 from shade import meta
 
-
-log = _log.setup_logging(__name__)
-
-
 _decorated_methods = []
 
 
+def _exc_clear():
+    """Because sys.exc_clear is gone in py3 and is not in six."""
+    if sys.version_info[0] == 2:
+        sys.exc_clear()
+
+
 def _iterate_timeout(timeout, message, wait=2):
     """Iterate and raise an exception on timeout.
 
@@ -42,8 +48,18 @@ def _iterate_timeout(timeout, message, wait=2):
     with <message>.
 
     """
+    log = _log.setup_logging('shade.iterate_timeout')
 
     try:
+        # None as a wait winds up flowing well in the per-resource cache
+        # flow. We could spread this logic around to all of the calling
+        # points, but just having this treat None as "I don't have a value"
+        # seems friendlier
+        if wait is None:
+            wait = 2
+        elif wait == 0:
+            # wait should be < timeout, unless timeout is None
+            wait = 0.1 if timeout is None else min(0.1, timeout)
         wait = float(wait)
     except ValueError:
         raise exc.OpenStackCloudException(
@@ -55,11 +71,40 @@ def _iterate_timeout(timeout, message, wait=2):
     while (timeout is None) or (time.time() < start + timeout):
         count += 1
         yield count
-        log.debug('Waiting {wait} seconds'.format(wait=wait))
+        log.debug('Waiting %s seconds', wait)
         time.sleep(wait)
     raise exc.OpenStackCloudTimeout(message)
 
 
+def _make_unicode(input):
+    """Turn an input into unicode unconditionally
+
+    :param input:
+       A unicode, string or other object
+    """
+    try:
+        if isinstance(input, unicode):
+            return input
+        if isinstance(input, str):
+            return input.decode('utf-8')
+        else:
+            # int, for example
+            return unicode(input)
+    except NameError:
+        # python3!
+        return str(input)
+
+
+def _dictify_resource(resource):
+    if isinstance(resource, list):
+        return [_dictify_resource(r) for r in resource]
+    else:
+        if hasattr(resource, 'toDict'):
+            return resource.toDict()
+        else:
+            return resource
+
+
 def _filter_list(data, name_or_id, filters):
     """Filter a list by name/ID and arbitrary meta data.
 
@@ -68,8 +113,9 @@ def _filter_list(data, name_or_id, filters):
         each dictionary contains an 'id' and 'name'
         key if a value for name_or_id is given.
     :param string name_or_id:
-        The name or ID of the entity being filtered.
-    :param dict filters:
+        The name or ID of the entity being filtered. Can be a glob pattern,
+        such as 'nb01*'.
+    :param filters:
         A dictionary of meta data to use for further filtering. Elements
         of this dictionary may, themselves, be dictionaries. Example::
 
@@ -79,19 +125,53 @@ def _filter_list(data, name_or_id, filters):
                   'gender': 'Female'
               }
             }
+        OR
+        A string containing a jmespath expression for further filtering.
     """
+    # The logger is shade.fmmatch to allow a user/operator to configure logging
+    # not to communicate about fnmatch misses (they shouldn't be too spammy,
+    # but one never knows)
+    log = _log.setup_logging('shade.fnmatch')
     if name_or_id:
+        # name_or_id might already be unicode
+        name_or_id = _make_unicode(name_or_id)
         identifier_matches = []
+        bad_pattern = False
+        try:
+            fn_reg = re.compile(fnmatch.translate(name_or_id))
+        except sre_constants.error:
+            # If the fnmatch re doesn't compile, then we don't care,
+            # but log it in case the user DID pass a pattern but did
+            # it poorly and wants to know what went wrong with their
+            # search
+            fn_reg = None
         for e in data:
-            e_id = str(e.get('id', None))
-            e_name = e.get('name', None)
-            if str(name_or_id) in (e_id, e_name):
+            e_id = _make_unicode(e.get('id', None))
+            e_name = _make_unicode(e.get('name', None))
+
+            if ((e_id and e_id == name_or_id) or
+                    (e_name and e_name == name_or_id)):
                 identifier_matches.append(e)
+            else:
+                # Only try fnmatch if we don't match exactly
+                if not fn_reg:
+                    # If we don't have a pattern, skip this, but set the flag
+                    # so that we log the bad pattern
+                    bad_pattern = True
+                    continue
+                if ((e_id and fn_reg.match(e_id)) or
+                        (e_name and fn_reg.match(e_name))):
+                    identifier_matches.append(e)
+        if not identifier_matches and bad_pattern:
+            log.debug("Bad pattern passed to fnmatch", exc_info=True)
         data = identifier_matches
 
     if not filters:
         return data
 
+    if isinstance(filters, six.string_types):
+        return jmespath.search(filters, data)
+
     def _dict_filter(f, d):
         if not d:
             return False
@@ -117,61 +197,53 @@ def _filter_list(data, name_or_id, filters):
     return filtered
 
 
-def _get_entity(func, name_or_id, filters):
+def _get_entity(cloud, resource, name_or_id, filters, **kwargs):
     """Return a single entity from the list returned by a given method.
 
-    :param callable func:
-        A function that takes `name_or_id` and `filters` as parameters
-        and returns a list of entities to filter.
+    :param object cloud:
+        The controller class (Example: the main OpenStackCloud object) .
+    :param string or callable resource:
+        The string that identifies the resource to use to lookup the
+        get_<>_by_id or search_<resource>s methods(Example: network)
+        or a callable to invoke.
     :param string name_or_id:
-        The name or ID of the entity being filtered or a dict
-    :param dict filters:
+        The name or ID of the entity being filtered or an object or dict.
+        If this is an object/dict with an 'id' attr/key, we return it and
+        bypass resource lookup.
+    :param filters:
         A dictionary of meta data to use for further filtering.
+        OR
+        A string containing a jmespath expression for further filtering.
+        Example:: "[?last_name==`Smith`] | [?other.gender]==`Female`]"
     """
+
     # Sometimes in the control flow of shade, we already have an object
     # fetched. Rather than then needing to pull the name or id out of that
     # object, pass it in here and rely on caching to prevent us from making
     # an additional call, it's simple enough to test to see if we got an
     # object and just short-circuit return it.
-    if hasattr(name_or_id, 'id'):
+
+    if (hasattr(name_or_id, 'id') or
+       (isinstance(name_or_id, dict) and 'id' in name_or_id)):
         return name_or_id
-    entities = func(name_or_id, filters)
-    if not entities:
-        return None
-    if len(entities) > 1:
-        raise exc.OpenStackCloudException(
-            "Multiple matches found for %s" % name_or_id)
-    return entities[0]
 
+    # If a uuid is passed short-circuit it calling the
+    # get_<resorce_name>_by_id method
+    if getattr(cloud, 'use_direct_get', False) and _is_uuid_like(name_or_id):
+        get_resource = getattr(cloud, 'get_%s_by_id' % resource, None)
+        if get_resource:
+            return get_resource(name_or_id)
 
-def normalize_servers(servers, cloud_name, region_name):
-    # Here instead of _utils because we need access to region and cloud
-    # name from the cloud object
-    ret = []
-    for server in servers:
-        ret.append(normalize_server(server, cloud_name, region_name))
-    return ret
-
-
-def normalize_server(server, cloud_name, region_name):
-    server.pop('links', None)
-    server['flavor'].pop('links', None)
-    # OpenStack can return image as a string when you've booted
-    # from volume
-    if str(server['image']) != server['image']:
-        server['image'].pop('links', None)
-
-    server['region'] = region_name
-    server['cloud'] = cloud_name
-
-    az = server.get('OS-EXT-AZ:availability_zone', None)
-    if az:
-        server['az'] = az
-
-    # Ensure volumes is always in the server dict, even if empty
-    server['volumes'] = []
-
-    return server
+    search = resource if callable(resource) else getattr(
+        cloud, 'search_%ss' % resource, None)
+    if search:
+        entities = search(name_or_id, filters, **kwargs)
+        if entities:
+            if len(entities) > 1:
+                raise exc.OpenStackCloudException(
+                    "Multiple matches found for %s" % name_or_id)
+            return entities[0]
+    return None
 
 
 def normalize_keystone_services(services):
@@ -196,132 +268,7 @@ def normalize_keystone_services(services):
             'enabled': service['enabled']
         }
         ret.append(new_service)
-    return meta.obj_list_to_dict(ret)
-
-
-def normalize_nova_secgroups(groups):
-    """Normalize the structure of nova security groups
-
-    This makes security group dicts, as returned from nova, look like the
-    security group dicts as returned from neutron. This does not make them
-    look exactly the same, but it's pretty close.
-
-    :param list groups: A list of security group dicts.
-
-    :returns: A list of normalized dicts.
-    """
-    ret = [{'id': g['id'],
-            'name': g['name'],
-            'description': g['description'],
-            'security_group_rules': normalize_nova_secgroup_rules(g['rules'])
-            } for g in groups]
-    return meta.obj_list_to_dict(ret)
-
-
-def normalize_nova_secgroup_rules(rules):
-    """Normalize the structure of nova security group rules
-
-    Note that nova uses -1 for non-specific port values, but neutron
-    represents these with None.
-
-    :param list rules: A list of security group rule dicts.
-
-    :returns: A list of normalized dicts.
-    """
-    ret = [{'id': r['id'],
-            'direction': 'ingress',
-            'ethertype': 'IPv4',
-            'port_range_min':
-                None if r['from_port'] == -1 else r['from_port'],
-            'port_range_max':
-                None if r['to_port'] == -1 else r['to_port'],
-            'protocol': r['ip_protocol'],
-            'remote_ip_prefix': r['ip_range'].get('cidr', None),
-            'security_group_id': r['parent_group_id']
-            } for r in rules]
-    return meta.obj_list_to_dict(ret)
-
-
-def normalize_nova_floating_ips(ips):
-    """Normalize the structure of Neutron floating IPs
-
-    Unfortunately, not all the Neutron floating_ip attributes are available
-    with Nova and not all Nova floating_ip attributes are available with
-    Neutron.
-    This function extract attributes that are common to Nova and Neutron
-    floating IP resource.
-    If the whole structure is needed inside shade, shade provides private
-    methods that returns "original" objects (e.g. _nova_allocate_floating_ip)
-
-    :param list ips: A list of Nova floating IPs.
-
-    :returns:
-        A list of normalized dicts with the following attributes::
-
-        [
-          {
-            "id": "this-is-a-floating-ip-id",
-            "fixed_ip_address": "192.0.2.10",
-            "floating_ip_address": "198.51.100.10",
-            "network": "this-is-a-net-or-pool-id",
-            "attached": True,
-            "status": "ACTIVE"
-          }, ...
-        ]
-
-    """
-    ret = [dict(
-        id=ip['id'],
-        fixed_ip_address=ip.get('fixed_ip'),
-        floating_ip_address=ip['ip'],
-        network=ip['pool'],
-        attached=(ip.get('instance_id') is not None and
-                  ip.get('instance_id') != ''),
-        status='ACTIVE'  # In neutrons terms, Nova floating IPs are always
-                         # ACTIVE
-    ) for ip in ips]
-    return meta.obj_list_to_dict(ret)
-
-
-def normalize_neutron_floating_ips(ips):
-    """Normalize the structure of Neutron floating IPs
-
-    Unfortunately, not all the Neutron floating_ip attributes are available
-    with Nova and not all Nova floating_ip attributes are available with
-    Neutron.
-    This function extract attributes that are common to Nova and Neutron
-    floating IP resource.
-    If the whole structure is needed inside shade, shade provides private
-    methods that returns "original" objects (e.g.
-    _neutron_allocate_floating_ip)
-
-    :param list ips: A list of Neutron floating IPs.
-
-    :returns:
-        A list of normalized dicts with the following attributes::
-
-        [
-          {
-            "id": "this-is-a-floating-ip-id",
-            "fixed_ip_address": "192.0.2.10",
-            "floating_ip_address": "198.51.100.10",
-            "network": "this-is-a-net-or-pool-id",
-            "attached": True,
-            "status": "ACTIVE"
-          }, ...
-        ]
-
-    """
-    ret = [dict(
-        id=ip['id'],
-        fixed_ip_address=ip.get('fixed_ip_address'),
-        floating_ip_address=ip['floating_ip_address'],
-        network=ip['floating_network_id'],
-        attached=(ip.get('port_id') is not None and
-                  ip.get('port_id') != ''),
-        status=ip['status']
-    ) for ip in ips]
-    return meta.obj_list_to_dict(ret)
+    return meta.obj_list_to_munch(ret)
 
 
 def localhost_supports_ipv6():
@@ -332,7 +279,10 @@ def localhost_supports_ipv6():
     IPv6 connectivity.
     """
 
-    return netifaces.AF_INET6 in netifaces.gateways()['default']
+    try:
+        return netifaces.AF_INET6 in netifaces.gateways()['default']
+    except AttributeError:
+        return False
 
 
 def normalize_users(users):
@@ -346,33 +296,10 @@ def normalize_users(users):
                                         user.get('tenantId')),
             domain_id=user.get('domain_id'),
             enabled=user.get('enabled'),
+            description=user.get('description')
         ) for user in users
     ]
-    return meta.obj_list_to_dict(ret)
-
-
-def normalize_volumes(volumes):
-    ret = []
-    for vol in volumes:
-        new_vol = vol.copy()
-        name = vol.get('name', vol.get('display_name'))
-        description = vol.get('description', vol.get('display_description'))
-        new_vol['name'] = name
-        new_vol['display_name'] = name
-        new_vol['description'] = description
-        new_vol['display_description'] = description
-        # For some reason, cinder v1 uses strings for bools for these fields.
-        # Cinder v2 uses booleans.
-        for field in ('bootable', 'multiattach'):
-            if field in new_vol and isinstance(new_vol[field],
-                                               six.string_types):
-                if new_vol[field] is not None:
-                    if new_vol[field].lower() == 'true':
-                        new_vol[field] = True
-                    elif new_vol[field].lower() == 'false':
-                        new_vol[field] = False
-        ret.append(new_vol)
-    return meta.obj_list_to_dict(ret)
+    return meta.obj_list_to_munch(ret)
 
 
 def normalize_domains(domains):
@@ -384,7 +311,7 @@ def normalize_domains(domains):
             enabled=domain.get('enabled'),
         ) for domain in domains
     ]
-    return meta.obj_list_to_dict(ret)
+    return meta.obj_list_to_munch(ret)
 
 
 def normalize_groups(domains):
@@ -397,7 +324,7 @@ def normalize_groups(domains):
             domain_id=domain.get('domain_id'),
         ) for domain in domains
     ]
-    return meta.obj_list_to_dict(ret)
+    return meta.obj_list_to_munch(ret)
 
 
 def normalize_role_assignments(assignments):
@@ -450,42 +377,15 @@ def normalize_role_assignments(assignments):
     return new_assignments
 
 
-def normalize_roles(roles):
-    """Normalize Identity roles."""
-    ret = [
+def normalize_flavor_accesses(flavor_accesses):
+    """Normalize Flavor access list."""
+    return [munch.Munch(
         dict(
-            id=role.get('id'),
-            name=role.get('name'),
-        ) for role in roles
+            flavor_id=acl.get('flavor_id'),
+            project_id=acl.get('project_id') or acl.get('tenant_id'),
+        )
+    ) for acl in flavor_accesses
     ]
-    return meta.obj_list_to_dict(ret)
-
-
-def normalize_stacks(stacks):
-    """ Normalize Stack Object """
-    for stack in stacks:
-        stack['name'] = stack['stack_name']
-    return stacks
-
-
-def normalize_flavors(flavors):
-    """ Normalize a list of flavor objects """
-    for flavor in flavors:
-        flavor.pop('links', None)
-        flavor.pop('NAME_ATTR', None)
-        flavor.pop('HUMAN_ID', None)
-        flavor.pop('human_id', None)
-        if 'extra_specs' not in flavor:
-            flavor['extra_specs'] = {}
-        ephemeral = flavor.pop('OS-FLV-EXT-DATA:ephemeral', 0)
-        is_public = flavor.pop('os-flavor-access:is_public', True)
-        # Make sure both the extension version and a sane version are present
-        flavor['OS-FLV-EXT-DATA:ephemeral'] = ephemeral
-        flavor['ephemeral'] = ephemeral
-        flavor['os-flavor-access:is_public'] = is_public
-        flavor['is_public'] = is_public
-
-    return flavors
 
 
 def valid_kwargs(*valid_args):
@@ -515,16 +415,19 @@ def valid_kwargs(*valid_args):
 
 
 def cache_on_arguments(*cache_on_args, **cache_on_kwargs):
+    _cache_name = cache_on_kwargs.pop('resource', None)
+
     def _inner_cache_on_arguments(func):
         def _cache_decorator(obj, *args, **kwargs):
-            the_method = obj._cache.cache_on_arguments(
+            the_method = obj._get_cache(_cache_name).cache_on_arguments(
                 *cache_on_args, **cache_on_kwargs)(
                     func.__get__(obj, type(obj)))
             return the_method(*args, **kwargs)
 
         def invalidate(obj, *args, **kwargs):
-            return obj._cache.cache_on_arguments()(func).invalidate(
-                *args, **kwargs)
+            return obj._get_cache(
+                _cache_name).cache_on_arguments()(func).invalidate(
+                    *args, **kwargs)
 
         _cache_decorator.invalidate = invalidate
         _cache_decorator.func = func
@@ -534,25 +437,6 @@ def cache_on_arguments(*cache_on_args, **cache_on_kwargs):
     return _inner_cache_on_arguments
 
 
-@contextlib.contextmanager
-def neutron_exceptions(error_message):
-    try:
-        yield
-    except neutron_exc.NotFound as e:
-        raise exc.OpenStackCloudResourceNotFound(
-            "{msg}: {exc}".format(msg=error_message, exc=str(e)))
-    except neutron_exc.NeutronClientException as e:
-        if e.status_code == 404:
-            raise exc.OpenStackCloudURINotFound(
-                "{msg}: {exc}".format(msg=error_message, exc=str(e)))
-        else:
-            raise exc.OpenStackCloudException(
-                "{msg}: {exc}".format(msg=error_message, exc=str(e)))
-    except Exception as e:
-        raise exc.OpenStackCloudException(
-            "{msg}: {exc}".format(msg=error_message, exc=str(e)))
-
-
 @contextlib.contextmanager
 def shade_exceptions(error_message=None):
     """Context manager for dealing with shade exceptions.
@@ -642,6 +526,60 @@ def safe_dict_max(key, data):
     return max_value
 
 
+def _call_client_and_retry(client, url, retry_on=None,
+                           call_retries=3, retry_wait=2,
+                           **kwargs):
+    """Method to provide retry operations.
+
+    Some APIs utilize HTTP errors on certian operations to indicate that
+    the resource is presently locked, and as such this mechanism provides
+    the ability to retry upon known error codes.
+
+    :param object client: The client method, such as:
+                          ``self.baremetal_client.post``
+    :param string url: The URL to perform the operation upon.
+    :param integer retry_on: A list of error codes that can be retried on.
+                             The method also supports a single integer to be
+                             defined.
+    :param integer call_retries: The number of times to retry the call upon
+                                 the error code defined by the 'retry_on'
+                                 parameter. Default: 3
+    :param integer retry_wait: The time in seconds to wait between retry
+                               attempts. Default: 2
+
+    :returns: The object returned by the client call.
+    """
+
+    # NOTE(TheJulia): This method, as of this note, does not have direct
+    # unit tests, although is fairly well tested by the tests checking
+    # retry logic in test_baremetal_node.py.
+    log = _log.setup_logging('shade.http')
+
+    if isinstance(retry_on, int):
+        retry_on = [retry_on]
+
+    count = 0
+    while (count < call_retries):
+        count += 1
+        try:
+            ret_val = client(url, **kwargs)
+        except exc.OpenStackCloudHTTPError as e:
+            if (retry_on is not None and
+                    e.response.status_code in retry_on):
+                log.debug('Received retryable error {err}, waiting '
+                          '{wait} seconds to retry', {
+                              'err': e.response.status_code,
+                              'wait': retry_wait
+                          })
+                time.sleep(retry_wait)
+                continue
+            else:
+                raise
+        # Break out of the loop, since the loop should only continue
+        # when we encounter a known connection error.
+        return ret_val
+
+
 def parse_range(value):
     """Parse a numerical range string.
 
@@ -740,3 +678,82 @@ def range_filter(data, key, range_exp):
             if int(d[key]) == val_range[1]:
                 filtered.append(d)
         return filtered
+
+
+def generate_patches_from_kwargs(operation, **kwargs):
+    """Given a set of parameters, returns a list with the
+    valid patch values.
+
+    :param string operation: The operation to perform.
+    :param list kwargs: Dict of parameters.
+
+    :returns: A list with the right patch values.
+    """
+    patches = []
+    for k, v in kwargs.items():
+        patch = {'op': operation,
+                 'value': v,
+                 'path': '/%s' % k}
+        patches.append(patch)
+    return sorted(patches)
+
+
+class FileSegment(object):
+    """File-like object to pass to requests."""
+
+    def __init__(self, filename, offset, length):
+        self.filename = filename
+        self.offset = offset
+        self.length = length
+        self.pos = 0
+        self._file = open(filename, 'rb')
+        self.seek(0)
+
+    def tell(self):
+        return self._file.tell() - self.offset
+
+    def seek(self, offset, whence=0):
+        if whence == 0:
+            self._file.seek(self.offset + offset, whence)
+        elif whence == 1:
+            self._file.seek(offset, whence)
+        elif whence == 2:
+            self._file.seek(self.offset + self.length - offset, 0)
+
+    def read(self, size=-1):
+        remaining = self.length - self.pos
+        if remaining <= 0:
+            return b''
+
+        to_read = remaining if size < 0 else min(size, remaining)
+        chunk = self._file.read(to_read)
+        self.pos += len(chunk)
+
+        return chunk
+
+    def reset(self):
+        self._file.seek(self.offset, 0)
+
+
+def _format_uuid_string(string):
+    return (string.replace('urn:', '')
+                  .replace('uuid:', '')
+                  .strip('{}')
+                  .replace('-', '')
+                  .lower())
+
+
+def _is_uuid_like(val):
+    """Returns validation of a value as a UUID.
+
+    :param val: Value to verify
+    :type val: string
+    :returns: bool
+
+    .. versionchanged:: 1.1.1
+       Support non-lowercase UUIDs.
+    """
+    try:
+        return str(uuid.UUID(val)).replace('-', '') == _format_uuid_string(val)
+    except (TypeError, ValueError, AttributeError):
+        return False
