@@ -37,7 +37,7 @@ class TestFlavor(base.BaseFunctionalTestCase):
 
     def _cleanup_flavors(self):
         exception_list = list()
-        for f in self.operator_cloud.list_flavors():
+        for f in self.operator_cloud.list_flavors(get_extra=False):
             if f['name'].startswith(self.new_item_name):
                 try:
                     self.operator_cloud.delete_flavor(f['id'])
@@ -72,7 +72,7 @@ class TestFlavor(base.BaseFunctionalTestCase):
         self.assertEqual(5, flavor['ephemeral'])
         self.assertIn('is_public', flavor)
         self.assertIn('os-flavor-access:is_public', flavor)
-        self.assertEqual(True, flavor['is_public'])
+        self.assertTrue(flavor['is_public'])
 
         for key in flavor_kwargs.keys():
             self.assertIn(key, flavor)
@@ -94,7 +94,7 @@ class TestFlavor(base.BaseFunctionalTestCase):
         self.operator_cloud.create_flavor(**public_kwargs)
         self.operator_cloud.create_flavor(**private_kwargs)
 
-        flavors = self.operator_cloud.list_flavors()
+        flavors = self.operator_cloud.list_flavors(get_extra=False)
 
         # Flavor list will include the standard devstack flavors. We just want
         # to make sure both of the flavors we just created are present.
@@ -114,7 +114,7 @@ class TestFlavor(base.BaseFunctionalTestCase):
         new_flavor = self.operator_cloud.create_flavor(**private_kwargs)
 
         # Validate the 'demo' user cannot see the new flavor
-        flavors = self.demo_cloud.search_flavors(priv_flavor_name)
+        flavors = self.user_cloud.search_flavors(priv_flavor_name)
         self.assertEqual(0, len(flavors))
 
         # We need the tenant ID for the 'demo' user
@@ -125,14 +125,20 @@ class TestFlavor(base.BaseFunctionalTestCase):
         self.operator_cloud.add_flavor_access(new_flavor['id'], project['id'])
 
         # Now see if the 'demo' user has access to it
-        flavors = self.demo_cloud.search_flavors(priv_flavor_name)
+        flavors = self.user_cloud.search_flavors(priv_flavor_name)
         self.assertEqual(1, len(flavors))
         self.assertEqual(priv_flavor_name, flavors[0]['name'])
 
+        # Now see if the 'demo' user has access to it without needing
+        #  the demo_cloud access.
+        acls = self.operator_cloud.list_flavor_access(new_flavor['id'])
+        self.assertEqual(1, len(acls))
+        self.assertEqual(project['id'], acls[0]['project_id'])
+
         # Now revoke the access and make sure we can't find it
         self.operator_cloud.remove_flavor_access(new_flavor['id'],
                                                  project['id'])
-        flavors = self.demo_cloud.search_flavors(priv_flavor_name)
+        flavors = self.user_cloud.search_flavors(priv_flavor_name)
         self.assertEqual(0, len(flavors))
 
     def test_set_unset_flavor_specs(self):
@@ -159,7 +165,7 @@ class TestFlavor(base.BaseFunctionalTestCase):
 
         # Unset the 'foo' value
         self.operator_cloud.unset_flavor_specs(mod_flavor['id'], ['foo'])
-        mod_flavor = self.operator_cloud.get_flavor(new_flavor['id'])
+        mod_flavor = self.operator_cloud.get_flavor_by_id(new_flavor['id'])
 
         # Verify 'foo' is unset and 'bar' is still set
         self.assertEqual({'bar': 'bbb'}, mod_flavor['extra_specs'])
