@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -36,7 +34,7 @@ class TestNetwork(base.BaseFunctionalTestCase):
         for network in self.operator_cloud.list_networks():
             if network['name'].startswith(self.network_name):
                 try:
-                    self.operator_cloud.delete_network(network['name'])
+                    self.operator_cloud.delete_network(network['id'])
                 except Exception as e:
                     exception_list.append(str(e))
                     continue
@@ -51,6 +49,22 @@ class TestNetwork(base.BaseFunctionalTestCase):
         self.assertFalse(net1['shared'])
         self.assertFalse(net1['router:external'])
         self.assertTrue(net1['admin_state_up'])
+        self.assertTrue(net1['port_security_enabled'])
+
+    def test_get_network_by_id(self):
+        net1 = self.operator_cloud.create_network(name=self.network_name)
+        self.assertIn('id', net1)
+        self.assertEqual(self.network_name, net1['name'])
+        self.assertFalse(net1['shared'])
+        self.assertFalse(net1['router:external'])
+        self.assertTrue(net1['admin_state_up'])
+
+        ret_net1 = self.operator_cloud.get_network_by_id(net1.id)
+        self.assertIn('id', ret_net1)
+        self.assertEqual(self.network_name, ret_net1['name'])
+        self.assertFalse(ret_net1['shared'])
+        self.assertFalse(ret_net1['router:external'])
+        self.assertTrue(ret_net1['admin_state_up'])
 
     def test_create_network_advanced(self):
         net1 = self.operator_cloud.create_network(
@@ -66,20 +80,36 @@ class TestNetwork(base.BaseFunctionalTestCase):
         self.assertFalse(net1['admin_state_up'])
 
     def test_create_network_provider_flat(self):
+        existing_public = self.operator_cloud.search_networks(
+            filters={'provider:network_type': 'flat'})
+        if existing_public:
+            self.skipTest('Physical network already allocated')
         net1 = self.operator_cloud.create_network(
             name=self.network_name,
             shared=True,
             provider={
-                'physical_network': 'private',
+                'physical_network': 'public',
                 'network_type': 'flat',
             }
         )
         self.assertIn('id', net1)
         self.assertEqual(self.network_name, net1['name'])
         self.assertEqual('flat', net1['provider:network_type'])
-        self.assertEqual('private', net1['provider:physical_network'])
+        self.assertEqual('public', net1['provider:physical_network'])
         self.assertIsNone(net1['provider:segmentation_id'])
 
+    def test_create_network_port_security_disabled(self):
+        net1 = self.operator_cloud.create_network(
+            name=self.network_name,
+            port_security_enabled=False,
+        )
+        self.assertIn('id', net1)
+        self.assertEqual(self.network_name, net1['name'])
+        self.assertTrue(net1['admin_state_up'])
+        self.assertFalse(net1['shared'])
+        self.assertFalse(net1['router:external'])
+        self.assertFalse(net1['port_security_enabled'])
+
     def test_list_networks_filtered(self):
         net1 = self.operator_cloud.create_network(name=self.network_name)
         self.assertIsNotNone(net1)
