@@ -21,55 +21,70 @@ from shade.tests.functional import base
 
 class TestRangeSearch(base.BaseFunctionalTestCase):
 
+    def _filter_m1_flavors(self, results):
+        """The m1 flavors are the original devstack flavors"""
+        new_results = []
+        for flavor in results:
+            if flavor['name'].startswith("m1."):
+                new_results.append(flavor)
+        return new_results
+
     def test_range_search_bad_range(self):
-        flavors = self.demo_cloud.list_flavors()
+        flavors = self.user_cloud.list_flavors(get_extra=False)
         self.assertRaises(
             exc.OpenStackCloudException,
-            self.demo_cloud.range_search, flavors, {"ram": "<1a0"})
+            self.user_cloud.range_search, flavors, {"ram": "<1a0"})
 
     def test_range_search_exact(self):
-        flavors = self.demo_cloud.list_flavors()
-        result = self.demo_cloud.range_search(flavors, {"ram": "4096"})
+        flavors = self.user_cloud.list_flavors(get_extra=False)
+        result = self.user_cloud.range_search(flavors, {"ram": "4096"})
         self.assertIsInstance(result, list)
+        # should only be 1 m1 flavor with 4096 ram
+        result = self._filter_m1_flavors(result)
         self.assertEqual(1, len(result))
         self.assertEqual("m1.medium", result[0]['name'])
 
     def test_range_search_min(self):
-        flavors = self.demo_cloud.list_flavors()
-        result = self.demo_cloud.range_search(flavors, {"ram": "MIN"})
+        flavors = self.user_cloud.list_flavors(get_extra=False)
+        result = self.user_cloud.range_search(flavors, {"ram": "MIN"})
         self.assertIsInstance(result, list)
         self.assertEqual(1, len(result))
-        self.assertEqual("m1.tiny", result[0]['name'])
+        # older devstack does not have cirros256
+        self.assertIn(result[0]['name'], ('cirros256', 'm1.tiny'))
 
     def test_range_search_max(self):
-        flavors = self.demo_cloud.list_flavors()
-        result = self.demo_cloud.range_search(flavors, {"ram": "MAX"})
+        flavors = self.user_cloud.list_flavors(get_extra=False)
+        result = self.user_cloud.range_search(flavors, {"ram": "MAX"})
         self.assertIsInstance(result, list)
         self.assertEqual(1, len(result))
         self.assertEqual("m1.xlarge", result[0]['name'])
 
     def test_range_search_lt(self):
-        flavors = self.demo_cloud.list_flavors()
-        result = self.demo_cloud.range_search(flavors, {"ram": "<4096"})
+        flavors = self.user_cloud.list_flavors(get_extra=False)
+        result = self.user_cloud.range_search(flavors, {"ram": "<1024"})
         self.assertIsInstance(result, list)
-        self.assertEqual(2, len(result))
-        flavor_names = [r['name'] for r in result]
-        self.assertIn("m1.tiny", flavor_names)
-        self.assertIn("m1.small", flavor_names)
+        # should only be 1 m1 flavor with <1024 ram
+        result = self._filter_m1_flavors(result)
+        self.assertEqual(1, len(result))
+        self.assertEqual("m1.tiny", result[0]['name'])
 
     def test_range_search_gt(self):
-        flavors = self.demo_cloud.list_flavors()
-        result = self.demo_cloud.range_search(flavors, {"ram": ">4096"})
+        flavors = self.user_cloud.list_flavors(get_extra=False)
+        result = self.user_cloud.range_search(flavors, {"ram": ">4096"})
         self.assertIsInstance(result, list)
+        # should only be 2 m1 flavors with >4096 ram
+        result = self._filter_m1_flavors(result)
         self.assertEqual(2, len(result))
         flavor_names = [r['name'] for r in result]
         self.assertIn("m1.large", flavor_names)
         self.assertIn("m1.xlarge", flavor_names)
 
     def test_range_search_le(self):
-        flavors = self.demo_cloud.list_flavors()
-        result = self.demo_cloud.range_search(flavors, {"ram": "<=4096"})
+        flavors = self.user_cloud.list_flavors(get_extra=False)
+        result = self.user_cloud.range_search(flavors, {"ram": "<=4096"})
         self.assertIsInstance(result, list)
+        # should only be 3 m1 flavors with <=4096 ram
+        result = self._filter_m1_flavors(result)
         self.assertEqual(3, len(result))
         flavor_names = [r['name'] for r in result]
         self.assertIn("m1.tiny", flavor_names)
@@ -77,9 +92,11 @@ class TestRangeSearch(base.BaseFunctionalTestCase):
         self.assertIn("m1.medium", flavor_names)
 
     def test_range_search_ge(self):
-        flavors = self.demo_cloud.list_flavors()
-        result = self.demo_cloud.range_search(flavors, {"ram": ">=4096"})
+        flavors = self.user_cloud.list_flavors(get_extra=False)
+        result = self.user_cloud.range_search(flavors, {"ram": ">=4096"})
         self.assertIsInstance(result, list)
+        # should only be 3 m1 flavors with >=4096 ram
+        result = self._filter_m1_flavors(result)
         self.assertEqual(3, len(result))
         flavor_names = [r['name'] for r in result]
         self.assertIn("m1.medium", flavor_names)
@@ -87,37 +104,38 @@ class TestRangeSearch(base.BaseFunctionalTestCase):
         self.assertIn("m1.xlarge", flavor_names)
 
     def test_range_search_multi_1(self):
-        flavors = self.demo_cloud.list_flavors()
-        result = self.demo_cloud.range_search(
+        flavors = self.user_cloud.list_flavors(get_extra=False)
+        result = self.user_cloud.range_search(
             flavors, {"ram": "MIN", "vcpus": "MIN"})
         self.assertIsInstance(result, list)
         self.assertEqual(1, len(result))
-        self.assertEqual("m1.tiny", result[0]['name'])
+        # older devstack does not have cirros256
+        self.assertIn(result[0]['name'], ('cirros256', 'm1.tiny'))
 
     def test_range_search_multi_2(self):
-        flavors = self.demo_cloud.list_flavors()
-        result = self.demo_cloud.range_search(
-            flavors, {"ram": "<8192", "vcpus": "MIN"})
+        flavors = self.user_cloud.list_flavors(get_extra=False)
+        result = self.user_cloud.range_search(
+            flavors, {"ram": "<1024", "vcpus": "MIN"})
         self.assertIsInstance(result, list)
-        self.assertEqual(2, len(result))
+        result = self._filter_m1_flavors(result)
+        self.assertEqual(1, len(result))
         flavor_names = [r['name'] for r in result]
-        # All of these should have 1 vcpu
         self.assertIn("m1.tiny", flavor_names)
-        self.assertIn("m1.small", flavor_names)
 
     def test_range_search_multi_3(self):
-        flavors = self.demo_cloud.list_flavors()
-        result = self.demo_cloud.range_search(
+        flavors = self.user_cloud.list_flavors(get_extra=False)
+        result = self.user_cloud.range_search(
             flavors, {"ram": ">=4096", "vcpus": "<6"})
         self.assertIsInstance(result, list)
+        result = self._filter_m1_flavors(result)
         self.assertEqual(2, len(result))
         flavor_names = [r['name'] for r in result]
         self.assertIn("m1.medium", flavor_names)
         self.assertIn("m1.large", flavor_names)
 
     def test_range_search_multi_4(self):
-        flavors = self.demo_cloud.list_flavors()
-        result = self.demo_cloud.range_search(
+        flavors = self.user_cloud.list_flavors(get_extra=False)
+        result = self.user_cloud.range_search(
             flavors, {"ram": ">=4096", "vcpus": "MAX"})
         self.assertIsInstance(result, list)
         self.assertEqual(1, len(result))
