@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -111,9 +109,39 @@ class TestRouter(base.BaseFunctionalTestCase):
         self.assertEqual(net1['id'], ext_gw_info['network_id'])
         self.assertTrue(ext_gw_info['enable_snat'])
 
+    def test_create_router_project(self):
+        project = self.operator_cloud.get_project('demo')
+        self.assertIsNotNone(project)
+        proj_id = project['id']
+        net1_name = self.network_prefix + '_net1'
+        net1 = self.operator_cloud.create_network(
+            name=net1_name, external=True, project_id=proj_id)
+
+        router_name = self.router_prefix + '_create_project'
+        router = self.operator_cloud.create_router(
+            name=router_name,
+            admin_state_up=True,
+            ext_gateway_net_id=net1['id'],
+            project_id=proj_id
+        )
+
+        for field in EXPECTED_TOPLEVEL_FIELDS:
+            self.assertIn(field, router)
+
+        ext_gw_info = router['external_gateway_info']
+        for field in EXPECTED_GW_INFO_FIELDS:
+            self.assertIn(field, ext_gw_info)
+
+        self.assertEqual(router_name, router['name'])
+        self.assertEqual('ACTIVE', router['status'])
+        self.assertEqual(proj_id, router['tenant_id'])
+        self.assertEqual(net1['id'], ext_gw_info['network_id'])
+        self.assertTrue(ext_gw_info['enable_snat'])
+
     def _create_and_verify_advanced_router(self,
                                            external_cidr,
                                            external_gateway_ip=None):
+        # external_cidr must be passed in as unicode (u'')
         # NOTE(Shrews): The arguments are needed because these tests
         # will run in parallel and we want to make sure that each test
         # is using different resources to prevent race conditions.
@@ -126,7 +154,7 @@ class TestRouter(base.BaseFunctionalTestCase):
             gateway_ip=external_gateway_ip
         )
 
-        ip_net = ipaddress.IPv4Network(unicode(external_cidr))
+        ip_net = ipaddress.IPv4Network(external_cidr)
         last_ip = str(list(ip_net.hosts())[-1])
 
         router_name = self.router_prefix + '_create_advanced'
@@ -164,11 +192,11 @@ class TestRouter(base.BaseFunctionalTestCase):
         return router
 
     def test_create_router_advanced(self):
-        self._create_and_verify_advanced_router(external_cidr='10.2.2.0/24')
+        self._create_and_verify_advanced_router(external_cidr=u'10.2.2.0/24')
 
     def test_add_remove_router_interface(self):
         router = self._create_and_verify_advanced_router(
-            external_cidr='10.3.3.0/24')
+            external_cidr=u'10.3.3.0/24')
         net_name = self.network_prefix + '_intnet1'
         sub_name = self.subnet_prefix + '_intsub1'
         net = self.operator_cloud.create_network(name=net_name)
@@ -194,7 +222,7 @@ class TestRouter(base.BaseFunctionalTestCase):
 
     def test_list_router_interfaces(self):
         router = self._create_and_verify_advanced_router(
-            external_cidr='10.5.5.0/24')
+            external_cidr=u'10.5.5.0/24')
         net_name = self.network_prefix + '_intnet1'
         sub_name = self.subnet_prefix + '_intsub1'
         net = self.operator_cloud.create_network(name=net_name)
@@ -229,7 +257,7 @@ class TestRouter(base.BaseFunctionalTestCase):
 
     def test_update_router_name(self):
         router = self._create_and_verify_advanced_router(
-            external_cidr='10.7.7.0/24')
+            external_cidr=u'10.7.7.0/24')
 
         new_name = self.router_prefix + '_update_name'
         updated = self.operator_cloud.update_router(
@@ -248,9 +276,34 @@ class TestRouter(base.BaseFunctionalTestCase):
         self.assertEqual(router['external_gateway_info'],
                          updated['external_gateway_info'])
 
+    def test_update_router_routes(self):
+        router = self._create_and_verify_advanced_router(
+            external_cidr=u'10.7.7.0/24')
+
+        routes = [{
+            "destination": "10.7.7.0/24",
+            "nexthop": "10.7.7.99"
+        }]
+
+        updated = self.operator_cloud.update_router(
+            router['id'], routes=routes)
+        self.assertIsNotNone(updated)
+
+        for field in EXPECTED_TOPLEVEL_FIELDS:
+            self.assertIn(field, updated)
+
+        # Name is the only change we expect
+        self.assertEqual(routes, updated['routes'])
+
+        # Validate nothing else changed
+        self.assertEqual(router['status'], updated['status'])
+        self.assertEqual(router['admin_state_up'], updated['admin_state_up'])
+        self.assertEqual(router['external_gateway_info'],
+                         updated['external_gateway_info'])
+
     def test_update_router_admin_state(self):
         router = self._create_and_verify_advanced_router(
-            external_cidr='10.8.8.0/24')
+            external_cidr=u'10.8.8.0/24')
 
         updated = self.operator_cloud.update_router(
             router['id'], admin_state_up=True)
@@ -272,7 +325,7 @@ class TestRouter(base.BaseFunctionalTestCase):
 
     def test_update_router_ext_gw_info(self):
         router = self._create_and_verify_advanced_router(
-            external_cidr='10.9.9.0/24')
+            external_cidr=u'10.9.9.0/24')
 
         # create a new subnet
         existing_net_id = router['external_gateway_info']['network_id']
