@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -19,12 +17,11 @@ test_users
 Functional tests for `shade` user methods.
 """
 
-from shade import operator_cloud
 from shade import OpenStackCloudException
 from shade.tests.functional import base
 
 
-class TestUsers(base.BaseFunctionalTestCase):
+class TestUsers(base.KeystoneBaseFunctionalTestCase):
     def setUp(self):
         super(TestUsers, self).setUp()
         self.user_prefix = self.getUniqueString('user')
@@ -67,6 +64,10 @@ class TestUsers(base.BaseFunctionalTestCase):
         users = self.operator_cloud.search_users(filters={'enabled': True})
         self.assertIsNotNone(users)
 
+    def test_search_users_jmespath(self):
+        users = self.operator_cloud.search_users(filters="[?enabled]")
+        self.assertIsNotNone(users)
+
     def test_create_user(self):
         user_name = self.user_prefix + '_create'
         user_email = 'nobody@nowhere.com'
@@ -125,9 +126,21 @@ class TestUsers(base.BaseFunctionalTestCase):
         self.assertEqual(user_name, new_user['name'])
         self.assertEqual(user_email, new_user['email'])
         self.assertTrue(new_user['enabled'])
-        self.assertIsNotNone(operator_cloud(
-            username=user_name, password='new_secret',
-            auth_url=self.operator_cloud.auth['auth_url']).keystone_client)
+        self.assertTrue(self.operator_cloud.grant_role(
+            'member', user=user['id'], project='demo', wait=True))
+        self.addCleanup(
+            self.operator_cloud.revoke_role,
+            'member', user=user['id'], project='demo', wait=True)
+
+        new_cloud = self.operator_cloud.connect_as(
+            user_id=user['id'],
+            password='new_secret',
+            project_name='demo')
+
+        self.assertIsNotNone(new_cloud)
+        location = new_cloud.current_location
+        self.assertEqual(location['project']['name'], 'demo')
+        self.assertIsNotNone(new_cloud.service_catalog)
 
     def test_users_and_groups(self):
         i_ver = self.operator_cloud.cloud_config.get_api_version('identity')
