@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -18,15 +16,13 @@ import os_client_config
 
 from os_client_config import exceptions as occ_exc
 
-from shade import _utils
 from shade import exc
 from shade import inventory
-from shade import meta
 from shade.tests import fakes
 from shade.tests.unit import base
 
 
-class TestInventory(base.TestCase):
+class TestInventory(base.RequestsMockTestCase):
 
     def setUp(self):
         super(TestInventory, self).setUp()
@@ -103,10 +99,9 @@ class TestInventory(base.TestCase):
 
         inv = inventory.OpenStackInventory()
 
-        server = _utils.normalize_server(
-            meta.obj_to_dict(fakes.FakeServer(
-                '1234', 'test', 'ACTIVE', addresses={})),
-            region_name='', cloud_name='')
+        server = self.cloud._normalize_server(
+            fakes.make_fake_server(
+                '1234', 'test', 'ACTIVE', addresses={}))
         self.assertIsInstance(inv.clouds, list)
         self.assertEqual(1, len(inv.clouds))
         inv.clouds[0].list_servers.return_value = [server]
@@ -147,9 +142,3 @@ class TestInventory(base.TestCase):
 
         ret = inv.get_host('server_id')
         self.assertEqual(server, ret)
-
-    @mock.patch("shade.inventory.OpenStackInventory.search_hosts")
-    def test_get_host_no_detail(self, mock_search):
-        inv = inventory.OpenStackInventory()
-        inv.get_host('server_id', expand=False)
-        mock_search.assert_called_once_with('server_id', None, expand=False)
