@@ -13,6 +13,9 @@
 # limitations under the License.
 
 
+import concurrent.futures
+import mock
+
 from shade import task_manager
 from shade.tests.unit import base
 
@@ -21,42 +24,51 @@ class TestException(Exception):
     pass
 
 
-class TestTask(task_manager.Task):
+class TaskTest(task_manager.Task):
     def main(self, client):
         raise TestException("This is a test exception")
 
 
-class TestTaskGenerator(task_manager.Task):
+class TaskTestGenerator(task_manager.Task):
     def main(self, client):
         yield 1
 
 
-class TestTaskInt(task_manager.Task):
+class TaskTestInt(task_manager.Task):
     def main(self, client):
         return int(1)
 
 
-class TestTaskFloat(task_manager.Task):
+class TaskTestFloat(task_manager.Task):
     def main(self, client):
         return float(2.0)
 
 
-class TestTaskStr(task_manager.Task):
+class TaskTestStr(task_manager.Task):
     def main(self, client):
         return "test"
 
 
-class TestTaskBool(task_manager.Task):
+class TaskTestBool(task_manager.Task):
     def main(self, client):
         return True
 
 
-class TestTaskSet(task_manager.Task):
+class TaskTestSet(task_manager.Task):
     def main(self, client):
         return set([1, 2])
 
 
-class TestTaskManager(base.TestCase):
+class TaskTestAsync(task_manager.Task):
+    def __init__(self):
+        super(task_manager.Task, self).__init__()
+        self.run_async = True
+
+    def main(self, client):
+        pass
+
+
+class TestTaskManager(base.RequestsMockTestCase):
 
     def setUp(self):
         super(TestTaskManager, self).setUp()
@@ -69,24 +81,29 @@ class TestTaskManager(base.TestCase):
         Specifically, we test if we get the same behaviour with all the
         configured interpreters (e.g. py27, p34, pypy, ...)
         """
-        self.assertRaises(TestException, self.manager.submitTask, TestTask())
+        self.assertRaises(TestException, self.manager.submit_task, TaskTest())
 
     def test_dont_munchify_int(self):
-        ret = self.manager.submitTask(TestTaskInt())
+        ret = self.manager.submit_task(TaskTestInt())
         self.assertIsInstance(ret, int)
 
     def test_dont_munchify_float(self):
-        ret = self.manager.submitTask(TestTaskFloat())
+        ret = self.manager.submit_task(TaskTestFloat())
         self.assertIsInstance(ret, float)
 
     def test_dont_munchify_str(self):
-        ret = self.manager.submitTask(TestTaskStr())
+        ret = self.manager.submit_task(TaskTestStr())
         self.assertIsInstance(ret, str)
 
     def test_dont_munchify_bool(self):
-        ret = self.manager.submitTask(TestTaskBool())
+        ret = self.manager.submit_task(TaskTestBool())
         self.assertIsInstance(ret, bool)
 
     def test_dont_munchify_set(self):
-        ret = self.manager.submitTask(TestTaskSet())
+        ret = self.manager.submit_task(TaskTestSet())
         self.assertIsInstance(ret, set)
+
+    @mock.patch.object(concurrent.futures.ThreadPoolExecutor, 'submit')
+    def test_async(self, mock_submit):
+        self.manager.submit_task(TaskTestAsync())
+        self.assertTrue(mock_submit.called)
